@@ -25,15 +25,14 @@
 
 #include <klocale.h>
 #include <kde_file.h>
- #include <KLocalizedString>
 #include <kio/global.h>
 #include <kio/renamedialog.h>
 #include <kio/job.h>
 #include <kio/chmodjob.h>
 #include <kio/copyjob.h>
 #include <kio/deletejob.h>
-#include <qdebug.h>
-#include <QUrl>
+#include <kdebug.h>
+#include <kurl.h>
 #include <kdirnotify.h>
 #include <kglobal.h>
 #include <kstandarddirs.h>
@@ -59,7 +58,6 @@
 #include <solid/device.h>
 #include <solid/block.h>
 #include <solid/storageaccess.h>
-#include <QStandardPaths>
 
 TrashImpl::TrashImpl() :
     QObject(),
@@ -75,7 +73,7 @@ TrashImpl::TrashImpl() :
     if ( KDE_lstat( QFile::encodeName( QDir::homePath() ), &buff ) == 0 ) {
         m_homeDevice = buff.st_dev;
     } else {
-        qDebug() << "Should never happen: couldn't stat $HOME " << strerror( errno ) << endl;
+        kError() << "Should never happen: couldn't stat $HOME " << strerror( errno ) << endl;
     }
 }
 
@@ -120,10 +118,10 @@ int TrashImpl::testDir( const QString &_name ) const
     if ( !ok )
     {
         //KMessageBox::sorry( 0, i18n( "Could not create directory %1. Check for permissions." ).arg( name ) );
-        qWarning() << "could not create" << name ;
+        kWarning() << "could not create" << name ;
         return KIO::ERR_COULD_NOT_MKDIR;
     } else {
-        qDebug() << name << "created.";
+        kDebug() << name << "created.";
     }
   }
   else // exists already
@@ -144,9 +142,9 @@ bool TrashImpl::init()
     // see also kdesktop/init.cc for first time initialization
     m_initStatus = InitError;
     // $XDG_DATA_HOME/Trash, i.e. ~/.local/share/Trash by default.
-    const QString xdgDataDir = QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + QLatin1Char('/');
-   if ( !QDir().mkpath( xdgDataDir ) ) {
-        qWarning() << "failed to create " << xdgDataDir ;
+    const QString xdgDataDir = KGlobal::dirs()->localxdgdatadir();
+    if ( !KStandardDirs::makeDir( xdgDataDir, 0700 ) ) {
+        kWarning() << "failed to create " << xdgDataDir ;
         return false;
     }
 
@@ -166,15 +164,15 @@ bool TrashImpl::init()
     }
     m_trashDirectories.insert( 0, trashDir );
     m_initStatus = InitOK;
-    qDebug() << "initialization OK, home trash dir: " << trashDir;
+    kDebug() << "initialization OK, home trash dir: " << trashDir;
     return true;
 }
 
 void TrashImpl::migrateOldTrash()
 {
-    qDebug() ;
+    kDebug() ;
 
-    KConfigGroup g( KSharedConfig::openConfig(), "Paths" );
+    KConfigGroup g( KGlobal::config(), "Paths" );
     const QString oldTrashDir = g.readPathEntry( "Trash", QString() );
 
     if ( oldTrashDir.isEmpty() )
@@ -192,29 +190,29 @@ void TrashImpl::migrateOldTrash()
         int trashId;
         QString fileId;
         if ( !createInfo( srcPath, trashId, fileId ) ) {
-            qWarning() << "Trash migration: failed to create info for " << srcPath ;
+            kWarning() << "Trash migration: failed to create info for " << srcPath ;
             allOK = false;
         } else {
             bool ok = moveToTrash( srcPath, trashId, fileId );
             if ( !ok ) {
                 (void)deleteInfo( trashId, fileId );
-                qWarning() << "Trash migration: failed to create info for " << srcPath ;
+                kWarning() << "Trash migration: failed to create info for " << srcPath ;
                 allOK = false;
             } else {
-                qDebug() << "Trash migration: moved " << srcPath;
+                kDebug() << "Trash migration: moved " << srcPath;
             }
         }
     }
     if ( allOK ) {
         // We need to remove the old one, otherwise the desktop will have two trashcans...
-        qDebug() << "Trash migration: all OK, removing old trash directory";
+        kDebug() << "Trash migration: all OK, removing old trash directory";
         synchronousDel( oldTrashDir, false, true );
     }
 }
 
 bool TrashImpl::createInfo( const QString& origPath, int& trashId, QString& fileId )
 {
-    qDebug() << origPath;
+    kDebug() << origPath;
     // Check source
     const QByteArray origPath_c( QFile::encodeName( origPath ) );
     /*
@@ -236,29 +234,28 @@ bool TrashImpl::createInfo( const QString& origPath, int& trashId, QString& file
     // Choose destination trash
     trashId = findTrashDirectory( origPath );
     if ( trashId < 0 ) {
-        qWarning() << "OUCH - internal error, TrashImpl::findTrashDirectory returned " << trashId ;
+        kWarning() << "OUCH - internal error, TrashImpl::findTrashDirectory returned " << trashId ;
         return false; // ### error() needed?
     }
-    qDebug() << "trashing to " << trashId;
+    kDebug() << "trashing to " << trashId;
 
     // Grab original filename
-    QUrl url = QUrl::fromLocalFile( origPath );
+    KUrl url;
+    url.setPath( origPath );
     const QString origFileName = url.fileName();
 
     // Make destination file in info/
     url.setPath( infoPath( trashId, origFileName ) ); // we first try with origFileName
-    QUrl baseDirectory = QUrl::fromLocalFile( url.path() );
+    KUrl baseDirectory;
+    baseDirectory.setPath( url.directory() );
     // Here we need to use O_EXCL to avoid race conditions with other kioslave processes
     int fd = 0;
-    QString fileName;
     do {
-        qDebug() << "trying to create " << url.path() ;
+        kDebug() << "trying to create " << url.path() ;
         fd = KDE_open( QFile::encodeName( url.path() ), O_WRONLY | O_CREAT | O_EXCL, 0600 );
         if ( fd < 0 ) {
-            if (errno == EEXIST ) {
-                fileName = url.fileName();
-                url = url.adjusted(QUrl::RemoveFilename);
-                url.setPath(url.path() + KIO::RenameDialog::suggestName( baseDirectory,  fileName)  );
+            if ( errno == EEXIST ) {
+                url.setFileName( KIO::RenameDialog::suggestName( baseDirectory, url.fileName() ) );
                 // and try again on the next iteration
             } else {
                 error( KIO::ERR_COULD_NOT_WRITE, url.path() );
@@ -303,7 +300,7 @@ bool TrashImpl::createInfo( const QString& origPath, int& trashId, QString& file
 
     ::fclose( file );
 
-    qDebug() << "info file created in trashId=" << trashId << " : " << fileId;
+    kDebug() << "info file created in trashId=" << trashId << " : " << fileId;
     return true;
 }
 
@@ -320,7 +317,7 @@ QString TrashImpl::makeRelativePath( const QString& topdir, const QString& path
         Q_ASSERT( rel[0] != QLatin1Char('/') );
         return rel;
     } else { // shouldn't happen...
-        qWarning() << "Couldn't make relative path for " << realPath << " (" << path << "), with topdir=" << topdir ;
+        kWarning() << "Couldn't make relative path for " << realPath << " (" << path << "), with topdir=" << topdir ;
         return realPath;
     }
 }
@@ -360,15 +357,12 @@ bool TrashImpl::deleteInfo( int trashId, const QString& fileId )
 
 bool TrashImpl::moveToTrash( const QString& origPath, int trashId, const QString& fileId )
 {
-    qDebug() <<"Trashing" << origPath<<trashId<<fileId;
+    kDebug() ;
     if ( !adaptTrashSize( origPath, trashId ) )
         return false;
 
     const qulonglong pathSize = DiscSpaceUtil::sizeOfPath( origPath );
 
-    TrashSizeCache trashSize( trashDirectoryPath( trashId ) );
-    trashSize.initialize();
-
     const QString dest = filesPath( trashId, fileId );
     if ( !move( origPath, dest ) ) {
         // Maybe the move failed due to no permissions to delete source.
@@ -380,7 +374,10 @@ bool TrashImpl::moveToTrash( const QString& origPath, int trashId, const QString
         return false;
     }
 
-    trashSize.add( pathSize );
+    if (QFileInfo(dest).isDir()) {
+        TrashSizeCache trashSize( trashDirectoryPath( trashId ) );
+        trashSize.add( fileId, pathSize );
+    }
 
     fileAdded();
     return true;
@@ -393,15 +390,11 @@ bool TrashImpl::moveFromTrash( const QString& dest, int trashId, const QString&
         src += QLatin1Char('/');
         src += relativePath;
     }
-    const qulonglong pathSize = DiscSpaceUtil::sizeOfPath( src );
-
-    TrashSizeCache trashSize( trashDirectoryPath( trashId ) );
-    trashSize.initialize();
-
     if ( !move( src, dest ) )
         return false;
 
-    trashSize.remove( pathSize );
+    TrashSizeCache trashSize( trashDirectoryPath( trashId ) );
+    trashSize.remove( fileId );
 
     return true;
 }
@@ -411,16 +404,16 @@ bool TrashImpl::move( const QString& src, const QString& dest )
     if ( directRename( src, dest ) ) {
         // This notification is done by KIO::moveAs when using the code below
         // But if we do a direct rename we need to do the notification ourselves
-        org::kde::KDirNotify::emitFilesAdded( QUrl::fromLocalFile(dest) );
+        org::kde::KDirNotify::emitFilesAdded( dest );
         return true;
     }
     if ( m_lastErrorCode != KIO::ERR_UNSUPPORTED_ACTION )
         return false;
 
-    QUrl urlSrc = QUrl::fromLocalFile(src);
-    QUrl urlDest = QUrl::fromLocalFile(dest);
-
-    qDebug() << urlSrc << " -> " << urlDest;
+    KUrl urlSrc, urlDest;
+    urlSrc.setPath( src );
+    urlDest.setPath( dest );
+    kDebug() << urlSrc << " -> " << urlDest;
     KIO::CopyJob* job = KIO::moveAs( urlSrc, urlDest, KIO::HideProgressInfo );
     job->setUiDelegate(0);
     connect( job, SIGNAL( result(KJob*) ),
@@ -432,28 +425,27 @@ bool TrashImpl::move( const QString& src, const QString& dest )
 
 void TrashImpl::jobFinished(KJob* job)
 {
-    qDebug() << " error=" << job->error() << job->errorText();
+    kDebug() << " error=" << job->error();
     error( job->error(), job->errorText() );
-
     emit leaveModality();
 }
 
 bool TrashImpl::copyToTrash( const QString& origPath, int trashId, const QString& fileId )
 {
-    qDebug() ;
+    kDebug() ;
     if ( !adaptTrashSize( origPath, trashId ) )
         return false;
 
     const qulonglong pathSize = DiscSpaceUtil::sizeOfPath( origPath );
 
-    TrashSizeCache trashSize( trashDirectoryPath( trashId ) );
-    trashSize.initialize();
-
     const QString dest = filesPath( trashId, fileId );
     if ( !copy( origPath, dest ) )
         return false;
 
-    trashSize.add( pathSize );
+    if (QFileInfo(dest).isDir()) {
+        TrashSizeCache trashSize( trashDirectoryPath( trashId ) );
+        trashSize.add( fileId, pathSize );
+    }
 
     fileAdded();
     return true;
@@ -473,9 +465,11 @@ bool TrashImpl::copy( const QString& src, const QString& dest )
 {
     // kio_file's copy() method is quite complex (in order to be fast), let's just call it...
     m_lastErrorCode = 0;
-    QUrl urlSrc = QUrl::fromLocalFile( src );
-    QUrl urlDest = QUrl::fromLocalFile( dest );
-    qDebug() << "copying " << src << " to " << dest;
+    KUrl urlSrc;
+    urlSrc.setPath( src );
+    KUrl urlDest;
+    urlDest.setPath( dest );
+    kDebug() << "copying " << src << " to " << dest;
     KIO::CopyJob* job = KIO::copyAs( urlSrc, urlDest, KIO::HideProgressInfo );
     job->setUiDelegate(0);
     connect( job, SIGNAL( result(KJob*) ),
@@ -487,7 +481,7 @@ bool TrashImpl::copy( const QString& src, const QString& dest )
 
 bool TrashImpl::directRename( const QString& src, const QString& dest )
 {
-    qDebug() << src << " -> " << dest;
+    kDebug() << src << " -> " << dest;
     if ( KDE_rename( QFile::encodeName( src ), QFile::encodeName( dest ) ) != 0 ) {
         if (errno == EXDEV) {
             error( KIO::ERR_UNSUPPORTED_ACTION, QString::fromLatin1("rename") );
@@ -533,8 +527,6 @@ bool TrashImpl::del( int trashId, const QString& fileId )
     QString info = infoPath(trashId, fileId);
     QString file = filesPath(trashId, fileId);
 
-    const qulonglong fileSize = DiscSpaceUtil::sizeOfPath( file );
-
     QByteArray info_c = QFile::encodeName(info);
 
     KDE_struct_stat buff;
@@ -546,13 +538,14 @@ bool TrashImpl::del( int trashId, const QString& fileId )
         return false;
     }
 
-    TrashSizeCache trashSize( trashDirectoryPath( trashId ) );
-    trashSize.initialize();
-
-    if ( !synchronousDel( file, true, QFileInfo(file).isDir() ) )
+    const bool isDir = QFileInfo(file).isDir();
+    if ( !synchronousDel( file, true, isDir ) )
         return false;
 
-    trashSize.remove( fileSize );
+    if (isDir) {
+        TrashSizeCache trashSize( trashDirectoryPath( trashId ) );
+        trashSize.remove( fileId );
+    }
 
     QFile::remove( info );
     fileRemoved();
@@ -563,11 +556,12 @@ bool TrashImpl::synchronousDel( const QString& path, bool setLastErrorCode, bool
 {
     const int oldErrorCode = m_lastErrorCode;
     const QString oldErrorMsg = m_lastErrorMessage;
-    QUrl url = QUrl::fromLocalFile( path );
+    KUrl url;
+    url.setPath( path );
     // First ensure that all dirs have u+w permissions,
     // otherwise we won't be able to delete files in them (#130780).
     if ( isDir ) {
-       qDebug() << "chmod'ing " << url;
+        kDebug() << "chmod'ing " << url;
         KFileItem fileItem( url, QString::fromLatin1("inode/directory"), KFileItem::Unknown );
         KFileItemList fileItemList;
         fileItemList.append( fileItem );
@@ -591,7 +585,7 @@ bool TrashImpl::synchronousDel( const QString& path, bool setLastErrorCode, bool
 
 bool TrashImpl::emptyTrash()
 {
-    qDebug() ;
+    kDebug() ;
     // The naive implementation "delete info and files in every trash directory"
     // breaks when deleted directories contain files owned by other users.
     // We need to ensure that the .trashinfo file is only removed when the
@@ -620,7 +614,7 @@ bool TrashImpl::emptyTrash()
             myErrorMsg = m_lastErrorMessage;
             // and remember not to remove this file
             unremoveableFiles.insert(filesPath);
-            qDebug() << "Unremoveable:" << filesPath;
+            kDebug() << "Unremoveable:" << filesPath;
         }
 
         TrashSizeCache trashSize( trashDirectoryPath( info.trashId ) );
@@ -638,7 +632,7 @@ bool TrashImpl::emptyTrash()
                 continue;
             const QString filePath = filesDir + QLatin1Char('/') + fileName;
             if (!unremoveableFiles.contains(filePath)) {
-                qWarning() << "Removing orphaned file" << filePath;
+                kWarning() << "Removing orphaned file" << filePath;
                 QFile::remove(filePath);
             }
         }
@@ -678,7 +672,7 @@ TrashImpl::TrashedFileInfoList TrashImpl::list()
             if ( fileName == QLatin1String(".") || fileName == QLatin1String("..") )
                 continue;
             if ( !fileName.endsWith( QLatin1String(".trashinfo") ) ) {
-                qWarning() << "Invalid info file found in " << infoPath << " : " << fileName ;
+                kWarning() << "Invalid info file found in " << infoPath << " : " << fileName ;
                 continue;
             }
             fileName.truncate( fileName.length() - 10 );
@@ -700,7 +694,7 @@ QStringList TrashImpl::listDir( const QString& physicalPath )
 
 bool TrashImpl::infoForFile( int trashId, const QString& fileId, TrashedFileInfo& info )
 {
-    qDebug() << trashId << " " << fileId;
+    kDebug() << trashId << " " << fileId;
     info.trashId = trashId; // easy :)
     info.fileId = fileId; // equally easy
     info.physicalPath = filesPath( trashId, fileId );
@@ -744,7 +738,7 @@ QString TrashImpl::physicalPath( int trashId, const QString& fileId, const QStri
 void TrashImpl::error( int e, const QString& s )
 {
     if ( e )
-        qDebug() << e << " " << s;
+        kDebug() << e << " " << s;
     m_lastErrorCode = e;
     m_lastErrorMessage = s;
 }
@@ -768,7 +762,7 @@ bool TrashImpl::isEmpty() const
             ep = KDE_readdir( dp ); // look for third file
             closedir( dp );
             if ( ep != 0 ) {
-                //qDebug() << ep->d_name << " in " << infoPath << " -> not empty";
+                //kDebug() << ep->d_name << " in " << infoPath << " -> not empty";
                 return false; // not empty
             }
         }
@@ -805,7 +799,7 @@ static int idForDevice(const Solid::Device& device)
 {
     const Solid::Block* block = device.as<Solid::Block>();
     if (block) {
-        qDebug() << "major=" << block->deviceMajor() << " minor=" << block->deviceMinor();
+        kDebug() << "major=" << block->deviceMajor() << " minor=" << block->deviceMinor();
         return block->deviceMajor()*1000 + block->deviceMinor();
     } else {
         // Not a block device. Maybe a NFS mount?
@@ -815,7 +809,7 @@ static int idForDevice(const Solid::Device& device)
 
 int TrashImpl::findTrashDirectory( const QString& origPath )
 {
-    qDebug() << origPath;
+    kDebug() << origPath;
     // First check if same device as $HOME, then we use the home trash right away.
     KDE_struct_stat buff;
     if ( KDE_lstat( QFile::encodeName( origPath ), &buff ) == 0
@@ -824,24 +818,24 @@ int TrashImpl::findTrashDirectory( const QString& origPath )
 
     KMountPoint::Ptr mp = KMountPoint::currentMountPoints().findByPath( origPath );
     if (!mp) {
-        qDebug() << "KMountPoint found no mount point for" << origPath;
+        kDebug() << "KMountPoint found no mount point for" << origPath;
         return 0;
     }
     QString mountPoint = mp->mountPoint();
     const QString trashDir = trashForMountPoint( mountPoint, true );
-    qDebug() << "mountPoint=" << mountPoint << " trashDir=" << trashDir;
+    kDebug() << "mountPoint=" << mountPoint << " trashDir=" << trashDir;
     if ( trashDir.isEmpty() )
         return 0; // no trash available on partition
     int id = idForTrashDirectory( trashDir );
     if ( id > -1 ) {
-        qDebug() << " known with id " << id;
+        kDebug() << " known with id " << id;
         return id;
     }
     // new trash dir found, register it
     // but we need stability in the trash IDs, so that restoring or asking
     // for properties works even kio_trash gets killed because idle.
 #if 0
-    qDebug() << "found " << trashDir;
+    kDebug() << "found " << trashDir;
     m_trashDirectories.insert( ++m_lastId, trashDir );
     if ( !mountPoint.endsWith( '/' ) )
         mountPoint += '/';
@@ -850,9 +844,9 @@ int TrashImpl::findTrashDirectory( const QString& origPath )
 #endif
 
     const QString query = QString::fromLatin1("[StorageAccess.accessible == true AND StorageAccess.filePath == '")+mountPoint+QString::fromLatin1("']");
-    //qDebug() << "doing solid query:" << query;
+    //kDebug() << "doing solid query:" << query;
     const QList<Solid::Device> lst = Solid::Device::listFromQuery(query);
-    //qDebug() << "got" << lst.count() << "devices";
+    //kDebug() << "got" << lst.count() << "devices";
     if ( lst.isEmpty() ) // not a device. Maybe some tmpfs mount for instance.
         return 0; // use the home trash instead
     // Pretend we got exactly one...
@@ -864,7 +858,7 @@ int TrashImpl::findTrashDirectory( const QString& origPath )
         return 0;
     }
     m_trashDirectories.insert( id, trashDir );
-    qDebug() << "found" << trashDir << "gave it id" << id;
+    kDebug() << "found" << trashDir << "gave it id" << id;
     if (!mountPoint.endsWith(QLatin1Char('/')))
         mountPoint += QLatin1Char('/');
     m_topDirectories.insert( id, mountPoint );
@@ -888,7 +882,7 @@ void TrashImpl::scanTrashDirectories() const
                     continue;
                 }
                 m_trashDirectories.insert( trashId, trashDir );
-                qDebug() << "found " << trashDir << " gave it id " << trashId;
+                kDebug() << "found " << trashDir << " gave it id " << trashId;
                 if (!topdir.endsWith(QLatin1Char('/')))
                     topdir += QLatin1Char('/');
                 m_topDirectories.insert( trashId, topdir );
@@ -937,13 +931,13 @@ QString TrashImpl::trashForMountPoint( const QString& topdir, bool createIfNeede
                      && (buff.st_mode & 0777) == 0700 ) { // rwx for user
                     return trashDir;
                 }
-                qDebug() << "Directory " << trashDir << " exists but didn't pass the security checks, can't use it";
+                kDebug() << "Directory " << trashDir << " exists but didn't pass the security checks, can't use it";
             }
             else if ( createIfNeeded && initTrashDirectory( trashDir_c ) ) {
                 return trashDir;
             }
         } else {
-            qDebug() << "Root trash dir " << rootTrashDir << " exists but didn't pass the security checks, can't use it";
+            kDebug() << "Root trash dir " << rootTrashDir << " exists but didn't pass the security checks, can't use it";
         }
     }
 
@@ -960,7 +954,7 @@ QString TrashImpl::trashForMountPoint( const QString& topdir, bool createIfNeede
             if ( checkTrashSubdirs( trashDir_c ) )
                 return trashDir;
         }
-        qDebug() << "Directory " << trashDir << " exists but didn't pass the security checks, can't use it";
+        kDebug() << "Directory " << trashDir << " exists but didn't pass the security checks, can't use it";
         // Exists, but not useable
         return QString();
     }
@@ -984,10 +978,10 @@ int TrashImpl::idForTrashDirectory( const QString& trashDir ) const
 
 bool TrashImpl::initTrashDirectory( const QByteArray& trashDir_c ) const
 {
-    qDebug() << trashDir_c;
+    kDebug() << trashDir_c;
     if ( KDE_mkdir( trashDir_c, 0700 ) != 0 )
         return false;
-    qDebug() ;
+    kDebug() ;
     // This trash dir will be useable only if the directory is owned by user.
     // In theory this is the case, but not on e.g. USB keys...
     uid_t uid = getuid();
@@ -1000,7 +994,7 @@ bool TrashImpl::initTrashDirectory( const QByteArray& trashDir_c ) const
         return checkTrashSubdirs( trashDir_c );
 
     } else {
-        qDebug() << trashDir_c << " just created, by it doesn't have the right permissions, probably some strange unsupported filesystem";
+        kDebug() << trashDir_c << " just created, by it doesn't have the right permissions, probably some strange unsupported filesystem";
         ::rmdir( trashDir_c );
         return false;
     }
@@ -1041,10 +1035,10 @@ QString TrashImpl::topDirectoryPath( int trashId ) const
 
 // Helper method. Creates a URL with the format trash:/trashid-fileid or
 // trash:/trashid-fileid/relativePath/To/File for a file inside a trashed directory.
-QUrl TrashImpl::makeURL( int trashId, const QString& fileId, const QString& relativePath )
+KUrl TrashImpl::makeURL( int trashId, const QString& fileId, const QString& relativePath )
 {
-    QUrl url;
-    url.setScheme(QString::fromLatin1("trash"));
+    KUrl url;
+    url.setProtocol(QString::fromLatin1("trash"));
     QString path = QString::fromLatin1("/");
     path += QString::number( trashId );
     path += QLatin1Char('-');
@@ -1059,11 +1053,14 @@ QUrl TrashImpl::makeURL( int trashId, const QString& fileId, const QString& rela
 
 // Helper method. Parses a trash URL with the URL scheme defined in makeURL.
 // The trash:/ URL itself isn't parsed here, must be caught by the caller before hand.
-bool TrashImpl::parseURL( const QUrl& url, int& trashId, QString& fileId, QString& relativePath )
+bool TrashImpl::parseURL( const KUrl& url, int& trashId, QString& fileId, QString& relativePath )
 {
-    if (url.scheme() != QLatin1String("trash"))
+    if (url.protocol() != QLatin1String("trash"))
         return false;
     const QString path = url.path();
+    if (path.isEmpty()) {
+        return false;
+    }
     int start = 0;
     if ( path[0] == QLatin1Char('/') ) // always true I hope
         start = 1;
@@ -1119,9 +1116,9 @@ bool TrashImpl::adaptTrashSize( const QString& origPath, int trashId )
         // calculate size of the files to be put into the trash
         qulonglong additionalSize = DiscSpaceUtil::sizeOfPath( origPath );
 
-        const TrashSizeCache trashSize( trashPath );
+        TrashSizeCache trashSize( trashPath );
         DiscSpaceUtil util(trashPath + QString::fromLatin1("/files/"));
-        if ( util.usage( trashSize.size() + additionalSize ) >= percent ) {
+        if ( util.usage( trashSize.calculateSize() + additionalSize ) >= percent ) {
             if ( actionType == 0 ) { // warn the user only
                 m_lastErrorCode = KIO::ERR_SLAVE_DEFINED;
                 m_lastErrorMessage = i18n( "The trash has reached its maximum size!\nCleanup the trash manually." );
@@ -1157,8 +1154,8 @@ bool TrashImpl::adaptTrashSize( const QString& origPath, int trashId )
 
                     del( trashId, info.fileName() ); // delete trashed file
 
-                    const TrashSizeCache trashSize( trashPath );
-                    if ( util.usage( trashSize.size() + additionalSize ) < percent ) // check whether we have enough space now
+                    TrashSizeCache trashSize( trashPath );
+                    if ( util.usage( trashSize.calculateSize() + additionalSize ) < percent ) // check whether we have enough space now
                          deleteFurther = false;
                 }
             }
@@ -1168,4 +1165,4 @@ bool TrashImpl::adaptTrashSize( const QString& origPath, int trashId )
     return true;
 }
 
-#include "moc_trashimpl.cpp"
+#include "trashimpl.moc"
